@@ -19,7 +19,7 @@
 
 name = "springheel"
 author = "gargargarrick"
-__version__ = "7.0.1"
+__version__ = "7.0.2"
 
 import springheel.classes
 import springheel.genchars
@@ -45,14 +45,14 @@ import springheel.copystuff
 import springheel.slugurl
 import springheel.wraptag
 import springheel.acopy
-from springheel.classes import *
+from springheel.classes import Site, Config, Tag, Strip, Comic, Chapter, Arrow, EXpage
 import springheel.addimg
 import springheel.renameimgs
 import springheel.webtoon
 import springheel.genstatline
 from springheel.tl import gettext as _
 
-import shutil, logging, textwrap, asyncio, glob
+import shutil, logging, textwrap, asyncio, glob, hashlib
 import configparser, os, datetime, sys
 from operator import itemgetter
 import html, json
@@ -90,8 +90,6 @@ def getTags(
         commas.
     this_strips_tags : list of Tag
         The Tags used by this strip.
-    all_tags : list of Tag
-        The updated list of all known tags.
     """
     tags_raw = meta["tags"]
     # Tags may be a list already in JSON mode
@@ -104,22 +102,13 @@ def getTags(
 
     for tag in tags_sep:
         tago = classes.Tag(name=tag)
-        tago.rurl = "tag-{tag_slug}.html".format(tag_slug=tago.slug)
-        label = "{tag_s}: {tag}".format(
-            tag_s=translated_strings["tag_s"], tag=tago.name
-        )
-        wrapped = (
-            """<a href="tag-{tag_slug}.html" aria-label="{label}">{tag}</a>""".format(
-                tag_slug=tago.slug, tag=tago.escaped, label=label
-            )
-        )
-        tago.link = wrapped
+        tago.gen_links(translated_strings)
         if tag not in set(n.name for n in all_tags):
             all_tags.append(tago)
         this_strips_tags.append(tago)
-        this_strips_wraps.append(wrapped)
+        this_strips_wraps.append(tago.link)
     tagline = ", ".join(this_strips_wraps)
-    return (tagline, this_strips_tags, all_tags)
+    return (tagline, this_strips_tags)
 
 
 def getComics(i_path: str, m_ext: str, t_ext: str) -> Tuple[List[Strip], dict]:
@@ -167,13 +156,13 @@ def getComics(i_path: str, m_ext: str, t_ext: str) -> Tuple[List[Strip], dict]:
     image_dimensions = dict()
 
     comics = []
-    # Find images that having matching metadata files.
+    # Find images that have matching metadata files.
     for i in tqdm(images):
         noext, ext = os.path.splitext(i)
         transcr = "".join([noext, ".", t_ext])
         meta = "".join([noext, ".", m_ext])
         basename = os.path.basename(i)
-        image_dimensions = retainDimensions(basename, i_path, image_dimensions)
+        retainDimensions(basename, i_path, image_dimensions)
         logmesg = _("{basename} dimensions: {dimensions}").format(
             basename=basename, dimensions=" x ".join(image_dimensions[basename])
         )
@@ -233,7 +222,7 @@ def getChapters(chapter_file: str) -> List[dict]:
         chapter_raws = f.read().splitlines()
     chapters = []
     for line in chapter_raws:
-        if line != "":
+        if line:
             split_line = line.split(" = ", 1)
             try:
                 d = {"num": int(split_line[0]), "title": split_line[1]}
@@ -479,7 +468,7 @@ def getDimensions(img_path: str) -> Tuple[str, str]:
     return str(width), str(height)
 
 
-def retainDimensions(img_fn: str, i_path: str, images: dict) -> dict:
+def retainDimensions(img_fn: str, i_path: str, images: dict) -> None:
     """
     Save image dimensions so they don't need to be recalculated.
 
@@ -492,11 +481,6 @@ def retainDimensions(img_fn: str, i_path: str, images: dict) -> dict:
     images : dict
         A dictionary mapping image filenames to (width, height) in
         pixels.
-
-    Returns
-    -------
-    dict
-        The image mapping dictionary, updated as needed.
     """
     try:
         w, h = images[img_fn]
@@ -504,7 +488,25 @@ def retainDimensions(img_fn: str, i_path: str, images: dict) -> dict:
     except KeyError:
         width, height = getDimensions(os.path.join(i_path, img_fn))
         images[img_fn] = (width, height)
-        return images
+
+
+async def imageChecksum(img_path: str) -> str:
+    """
+    Get the SHA-256 checksum for an image file.
+
+    Parameters
+    ----------
+    img_path : str
+        The path to the image.
+
+    Returns
+    -------
+    str
+        The checksum.
+    """
+    with open(img_path, "rb") as fin:
+        imgbytes = fin.read()
+    return hashlib.sha256(imgbytes).hexdigest()
 
 
 async def build():
@@ -534,6 +536,7 @@ async def build():
 
     site.jsonpoints = {
         "generated_on": starttime,
+        "springheel_version": __version__,
         **config.dict,
         "archive_page": "".join([site.config.base_url, "archive.html"]),
         "categories": [],
@@ -577,6 +580,7 @@ async def build():
     site.c_path = c_path
     site.i_path = i_path
     site.o_path = o_path
+    site.config.base_url = "/".join(i.strip("/") for i in [site.config.base_url, ""])
 
     try:
         zero_padding = site.config.zero_padding
@@ -724,127 +728,24 @@ async def build():
         meta, commentary, raw_comments = parsemeta.parseMetadata(
             file_name, translated_strings, json_mode
         )
-        # Page number
-        page = meta["page"]
-        all_pages_raw.add(page)
-        page_real_num, page_padded = padNum(page, zero_padding)
-        if not page_padded:
-            logging.error(_("Unable to parse page number {page}").format(page=page))
-            return False
-        all_pages_real.add(page_real_num)
-
-        strip.metadata = meta
-        strip.commentary = commentary
-        strip.raw_comments = raw_comments
-        strip.category = strip.metadata["category"]
-        strip.page = page
-        strip.page_real_num = page_real_num
-        strip.page_padded = page_padded
-        conf_file = os.path.join(i_path, strip.metadata["conf"])
-        site.images = retainDimensions(strip.imagef, i_path, site.images)
-        try:
-            (conf,) = [item for item in configs if item["category"] == strip.category]
-        except ValueError:
-            conf = springheel.parseconf.comicCParse(conf_file)
-            configs.append(conf)
-        strip.conf_c = conf
+        strip.populate(meta, commentary, raw_comments, configs, site)
+        all_pages_raw.add(strip.page)
+        all_pages_real.add(strip.page_real_num)
         all_page_nums.add(strip.page_padded)
+        retainDimensions(strip.imagef, i_path, site.images)
 
     cats_raw = set()
     cats_w_pages = []
+    comics = []
+    ccomics = []
     for strip in comics_base:
         cats_raw.add(strip.category)
     for cat in cats_raw:
-        known_pages = [page.page_padded for page in comics_base if page.category == cat]
-        known_pages_raw = [page.page for page in comics_base if page.category == cat]
-        if set([known_pages_raw.count(item) for item in known_pages_raw]) != {1}:
-            dupes = set(
-                [item for item in known_pages_raw if known_pages_raw.count(item) > 1]
-            )
-            logging.warning(
-                _("Duplicate page numbers found in category {cat}: {dupes}").format(
-                    cat=cat, dupes=", ".join(dupes)
-                )
-            )
-        known_pages_real = [
-            page.page_real_num for page in comics_base if page.category == cat
-        ]
-        known_pages.sort(key=mixNum)
-        known_pages_raw.sort()
-        known_pages_real.sort(key=realMixNum)
-        last_page, first_page = checkExtremes(known_pages)
-        cat_w_pages = {
-            "category": cat,
-            "last_page": last_page,
-            "first_page": first_page,
-            "known_pages": known_pages,
-            "known_pages_raw": known_pages_raw,
-            "known_pages_real": known_pages_real,
-        }
-        cats_w_pages.append(cat_w_pages)
-
-    comics = []
-    ccomics = []
-    author_dividers = {"&", "/", "+"}
-
-    for cat in cats_w_pages:
-        c = classes.Comic(cat["category"])
-        c.authors = []
-        c.last_page = cat["last_page"]
-        c.first_page = cat["first_page"]
-        c.known_pages = cat["known_pages"]
-        c.known_pages_raw = cat["known_pages_raw"]
-        c.known_pages_real = cat["known_pages_real"]
-        (conf,) = [item for item in configs if item["category"] == cat["category"]]
+        c = classes.Comic(cat)
+        (conf,) = [item for item in configs if item["category"] == cat]
         c.conf_c = conf
-        raw_author = conf["author"]
-        c.author = html.escape(raw_author)
-        if any([div in raw_author for div in author_dividers]):
-            raw_authors_split = springheel.splitseps.splitAtSeparators(
-                raw_author, author_dividers
-            )
-            for a in raw_authors_split:
-                cleaned_a = html.escape(a).strip()
-                if cleaned_a not in site.authors:
-                    site.authors.append(cleaned_a)
-                if cleaned_a not in c.authors:
-                    c.authors.append(cleaned_a)
-        else:
-            if c.author not in site.authors:
-                site.authors.append(c.author)
-            if c.author not in c.authors:
-                c.authors.append(c.author)
-        for key in {
-            "email",
-            "header",
-            "banner",
-            "language",
-            "mode",
-            "status",
-            "license_uri",
-            "about",
-        }:
-            try:
-                setattr(c, key, conf[key])
-            except KeyError:
-                logging.debug(
-                    _("No value for {key} in the config file for {cat}.").format(
-                        key=key, cat=cat["category"]
-                    )
-                )
-                continue
-        c.clicense = conf["license"]
-        try:
-            # Just to be absolutely sure the slug really is safe.
-            c.slug = slugurl.slugify_url(conf["slug"])
-        except KeyError:
-            c.slug = slugurl.slugify_url(cat["category"])
-        if conf["chapters"]:
-            c.chapters = True
-            c.chapters_file = os.path.join(i_path, conf["chapters"])
-        else:
-            c.chapters = False
-        c.desc = html.escape(conf["desc"])
+        c.authors = []
+        c.populate(site, comics_base)
 
         try:
             category_theme = conf["category_theme"]
@@ -867,18 +768,18 @@ async def build():
                     outf=os.path.relpath(new_banner_path), var=f"{c.category} banner"
                 )
             )
-        site.images = retainDimensions(c.header, i_path, site.images)
-        site.images = retainDimensions(c.banner, i_path, site.images)
+        retainDimensions(c.header, i_path, site.images)
+        retainDimensions(c.banner, i_path, site.images)
         c.headerw, c.headerh = site.images[c.header]
         c.bannerw, c.bannerh = site.images[c.banner]
 
         ccomics.append(c)
         if not [
-            item
-            for item in site.jsonpoints["categories"]
-            if item["category"] == cat["category"]
+            item for item in site.jsonpoints["categories"] if item["category"] == cat
         ]:
-            site.jsonpoints["categories"].append(cat)
+            site.jsonpoints["categories"].append(
+                {s: getattr(c, s) for s in c.__slots__ if hasattr(c, s)}
+            )
     ccomics.sort(key=lambda k: k.category)
 
     # Get other pages.
@@ -930,7 +831,7 @@ async def build():
                 translated_strings["no_transcript"], "p"
             )
             strip.transcript_c = no_transcr
-        (match,) = [item for item in ccomics if item.category == strip.category]
+        match = strip.get_matching(ccomics)
 
         strip.lang = strip.conf_c["language"]
         author_email = strip.conf_c["email"]
@@ -944,26 +845,7 @@ async def build():
         strip_id = match.known_pages.index(strip.page_padded)
 
         strip.title = html.escape(strip.metadata["title"])
-        if strip.metadata["author"] not in site.raw_authors:
-            # Multiple authors
-            i_raw_author = strip.metadata["author"]
-            site.raw_authors.add(i_raw_author)
-            strip.author = html.escape(i_raw_author)
-            if any([div in i_raw_author for div in author_dividers]):
-                strip.authors = []
-                raw_authors_split = springheel.splitseps.splitAtSeparators(
-                    i_raw_author, author_dividers
-                )
-                for a in raw_authors_split:
-                    cleaned_a = html.escape(a).strip()
-                    strip.authors.append(cleaned_a)
-            else:
-                strip.authors = [strip.author]
-            site.processed_authors[i_raw_author] = strip.authors
-        else:
-            strip.author = strip.metadata["author"]
-            strip.authors = site.processed_authors[strip.metadata["author"]]
-        strip.author_email = strip.metadata["email"]
+        strip.populate_authors(site)
         strip.title_slug = strip.metadata["title_slug"]
         strip.series_slug = match.slug
         strip.slugs = [strip.title_slug, strip.series_slug]
@@ -991,11 +873,11 @@ async def build():
         strip.width = width
 
         if match.chapters:
-            site.jsonpoints.setdefault("chapter_info", [])
+            site.jsonpoints.setdefault("chapter_info", dict())
             chapters_dicts = getChapters(match.chapters_file)
             match.chapters_dicts = chapters_dicts
-            if match.chapters_dicts not in site.jsonpoints["chapter_info"]:
-                site.jsonpoints["chapter_info"].append(match.chapters_dicts)
+            if match.category not in site.jsonpoints["chapter_info"].keys():
+                site.jsonpoints["chapter_info"][match.category] = match.chapters_dicts
             if not hasattr(match, "chapters_list"):
                 match.chapters_list = []
             for chapter in chapters_dicts:
@@ -1047,11 +929,7 @@ async def build():
             except KeyError:
                 try:
                     license_uri = match.conf_c["license_uri"]
-                    if (
-                        clicense.lower() == "public domain"
-                        or "publicdomain" in license_uri
-                    ):
-                        match.publicdomain = True
+                    if match.publicdomain:
                         # Creative Commons Public Domain Waiver
                         ccpdw = translated_strings["ccpdw"]
                         license_c = ccpdw.format(
@@ -1114,23 +992,13 @@ async def build():
             chapter = strip.metadata["chapter"]
         except KeyError:
             chapter = False
+        strip.sha256 = await imageChecksum(img_path)
 
-        (matching_cat,) = [
-            item
-            for item in cats_w_pages
-            if item["category"] == strip.metadata["category"]
-        ]
-        last_page = matching_cat["last_page"]
-        first_page = matching_cat["first_page"]
+        last_page = match.last_page
+        first_page = match.first_page
 
-        if page_real_num == last_page:
-            final = True
-        else:
-            final = False
-        if page_real_num == first_page:
-            first = True
-        else:
-            first = False
+        final = True if strip.page_real_num == last_page else False
+        first = True if strip.page_real_num == first_page else False
 
         strip.date = date
         date_format = translated_strings["date_format"]
@@ -1139,33 +1007,27 @@ async def build():
             date, translated_strings["strf_format"]
         ).format(y=date_format[0], m=date_format[1], d=date_format[2])
         strip.year = year
-        if chapter:
-            strip.chapter = chapter
-        else:
-            strip.chapter = False
+        strip.chapter = chapter if chapter else False
         if not alt_text:
             strip.alt_text = ""
             strip.figcaption = ""
         else:
             strip.alt_text = alt_text
-            strip.figcaption = (
-                '<figcaption tabindex="0">{alt_text}</figcaption>'.format(
-                    alt_text=alt_text
-                )
+            strip.figcaption = "<figcaption>{alt_text}</figcaption>".format(
+                alt_text=alt_text
             )
 
         try:
-            if match.category_theme:
-                style = match.category_theme
-            else:
-                style = site.config.site_style
+            style = (
+                match.category_theme if match.category_theme else site.config.site_style
+            )
         except AttributeError:
             style = site.config.site_style
 
         # Get arrow sizes
         for direction in {"first", "prev", "next", "last"}:
             arrow_fn = f"{style}_{direction}.png"
-            site.images = retainDimensions(arrow_fn, "arrows", site.images)
+            retainDimensions(arrow_fn, "arrows", site.images)
         navblock, linkrels = generatenav.navGen(
             site.config.navdirection,
             strip.page_padded,
@@ -1199,9 +1061,7 @@ async def build():
         strip.header_title = header_title
 
         try:
-            tagsline, these_tags, all_tags = getTags(
-                strip.metadata, all_tags, translated_strings
-            )
+            tagsline, these_tags = getTags(strip.metadata, all_tags, translated_strings)
             strip.tags = these_tags
             for tag in strip.tags:
                 (tag_match,) = [item for item in all_tags if item.name == tag.name]
@@ -1394,6 +1254,10 @@ async def build():
             except KeyError:
                 pass
         jsoncat["strips"].append(dictified_strip)
+        try:
+            del jsoncat["chapters_file"]
+        except KeyError:
+            pass
 
         skiplink = "#comic"
 
@@ -1516,6 +1380,10 @@ async def build():
     site_header_w, site_header_h = getDimensions(
         os.path.join(i_path, site.config.header_filename)
     )
+    for jsoncat in site.jsonpoints["categories"]:
+        jsoncat["authors"] = sorted(
+            list({author for strip in jsoncat["strips"] for author in strip["authors"]})
+        )
 
     # Comic-wide copyright statement
     for comic in ccomics:
@@ -1551,6 +1419,13 @@ async def build():
             except AttributeError:
                 license_c = comic.clicense
                 license_s = comic.clicense
+        comic.copyright_statement = license_c
+        jsoncat = [
+            item
+            for item in site.jsonpoints["categories"]
+            if item["category"] == comic.category
+        ][0]
+        jsoncat["copyright_statement"] = comic.copyright_statement
 
     # Sitewide copyright statement
     years = list(years)
@@ -1572,6 +1447,7 @@ async def build():
                 clicense=site.license,
             )
         site.copyright_statement = copyright_statement
+    site.jsonpoints["copyright_statement"] = site.copyright_statement
 
     for a_theme in themes:
         tasks.append(
@@ -1810,9 +1686,7 @@ async def build():
             for page in comic.pbp:
                 # Make sure there aren't unchaptered pages in chapter
                 if hasattr(page, "chapter") and page.chapter not in falses:
-                    match = [
-                        item for item in ccomics if item.category == page.category
-                    ][0]
+                    match = page.get_matching(ccomics)
                     cho = [
                         item
                         for item in match.chapters_list
@@ -2078,6 +1952,16 @@ async def build():
                 ch_bottom_nav = springheel.wraptag.wrapWithComment(
                     ch_navblock.format(boxlocation="botbox"), "BOTTOM NAVIGATION"
                 )
+                ch_main_classes = []
+                if site.config.navdirection == "rtl":
+                    ch_main_classes.append("rtl")
+                if modeclass:
+                    ch_main_classes.append("webtoon")
+                ch_main_class = (
+                    f""" class="{" ".join(ch_main_classes)}\""""
+                    if ch_main_classes
+                    else ""
+                )
 
                 chap_string = chap_template.format(
                     lang=comic.language,
@@ -2091,6 +1975,7 @@ async def build():
                     header=comic.header,
                     headerw=comic.headerw,
                     headerh=comic.headerh,
+                    mainclass=ch_main_class,
                     modeclass=modeclass,
                     category=comic.category_escaped,
                     top_site_nav=top_site_nav,
@@ -2231,9 +2116,10 @@ async def build():
     # Generate extras page if necessary.
     extras_j = os.path.join(i_path, "Extra.json")
     if extras_page and os.path.exists(extras_j):
-        extras, j = genextra.gen_extra(
+        extras, j, extra_copyq = genextra.gen_extra(
             i_path, o_path, extras_j, translated_strings, site.images
         )
+        tasks.append(genextra.copyExtras(extra_copyq))
 
         if j:
 
