@@ -19,6 +19,7 @@
 
 import html, configparser, datetime, os, logging
 import springheel.slugurl
+import springheel.splitseps
 from springheel.tl import gettext as _
 
 
@@ -272,15 +273,20 @@ class Tag:
     escaped : str
         An HTML-safe version of name (no &, <, or >).
     link : str
-        Hyperlink to the tag index URL.
+        HTML hyperlink to the tag index URL.
     name : str
         The name of the tag.
     rurl : str
-        HTML filename for the tag.
+        HTML filename for the tag index.
     slug : str
         name slugified for URLs.
     strips : list
         Strips to which the tag applies.
+
+    Methods
+    -------
+    gen_links:
+        Generate links for a Tag.
     """
 
     __slots__ = ["escaped", "link", "name", "rurl", "slug", "strips"]
@@ -296,9 +302,26 @@ class Tag:
         """
         self.name = name
         self.escaped = html.escape(name)
-        self.slug = springheel.slugurl.slugify_url(name)
+        self.slug = springheel.springheel.slugurl.slugify_url(name)
         self.strips = []
 
+    def gen_links(self, translated_strings: dict) -> None:
+        """
+        Generate links for a Tag.
+
+        Parameters
+        ----------
+        translated_strings : dict
+            The translation file contents for this site.
+        """
+        self.rurl = "tag-{tag_slug}.html".format(tag_slug=self.slug)
+        label = "{tag_s}: {tag}".format(
+            tag_s=translated_strings["tag_s"], tag=self.name
+        )
+        self.link = """<a href="{rurl}" aria-label="{label}">{tag}</a>""".format(
+            rurl=self.rurl, tag=self.escaped, label=label
+        )
+
 
 class Strip:
     """
@@ -399,6 +422,8 @@ class Strip:
         The strip's commentary without any HTML formatting, etc.
     series_slug : str
         URL-safe slug for the strip's category.
+    sha256 : str
+        SHA-256 checksum of the strip image. Included for API purposes.
     slug : str
         The HTML filename without the extension, for use as a slug.
     slugs : list
@@ -431,6 +456,15 @@ class Strip:
         The width of the strip image in pixels.
     year : int
         The year in which the strip was published.
+
+    Methods
+    -------
+    get_matching:
+        Find the Comic this Strip belongs to.
+    populate:
+        Add initial information to a comic Strip.
+    populate_authors:
+        Add author information for the Strip.
     """
 
     __slots__ = [
@@ -471,6 +505,7 @@ class Strip:
         "pieces",
         "raw_comments",
         "series_slug",
+        "sha256",
         "slug",
         "slugs",
         "source",
@@ -506,6 +541,99 @@ class Strip:
         self.transf = transf
         self.tags = []
 
+    def get_matching(self, ccomics):
+        """
+        Find the :class:`Comic` this Strip belongs to.
+
+        Parameters
+        ----------
+        ccomics : list
+            A list of Comic objects.
+
+        Returns
+        ----------
+        Comic
+            The matching Comic.
+        """
+        return [item for item in ccomics if item.category == self.category][0]
+
+    def populate(
+        self,
+        meta: dict,
+        commentary: str,
+        raw_comments: list,
+        configs: list,
+        site: Site,
+    ) -> None:
+        """
+        Add initial information to a comic Strip.
+
+        Parameters
+        ----------
+        meta : dict
+            Metadata about the strip.
+        commentary : str
+            The comic's commentary.
+        raw_comments : list of str
+            Raw commentary lines from the comic's metadata file.
+        configs : list of dict
+            Configuration files known to Springheel.
+        site : Site
+            The root site
+        """
+        self.page = meta["page"]
+        self.page_real_num, self.page_padded = springheel.padNum(
+            self.page, site.config.zero_padding
+        )
+        if not self.page_padded:
+            logging.error(
+                _("Unable to parse page number {page}").format(page=self.page)
+            )
+            return False
+        self.metadata = meta
+        self.commentary = commentary
+        self.raw_comments = raw_comments
+        self.category = self.metadata["category"]
+        conf_file = os.path.join(site.i_path, self.metadata["conf"])
+
+        try:
+            (conf,) = [item for item in configs if item["category"] == self.category]
+        except ValueError:
+            conf = springheel.parseconf.comicCParse(conf_file)
+            configs.append(conf)
+        self.conf_c = conf
+
+    def populate_authors(self, site: Site):
+        """
+        Add author information for the Strip.
+
+        Parameters
+        ----------
+        site : Site
+            The root site, which usually has author data.
+        """
+        author_dividers = {"&", "/", "+"}
+        if self.metadata["author"] not in site.raw_authors:
+            # Multiple authors
+            i_raw_author = self.metadata["author"]
+            site.raw_authors.add(i_raw_author)
+            self.author = html.escape(i_raw_author)
+            if any([div in i_raw_author for div in author_dividers]):
+                self.authors = []
+                raw_authors_split = springheel.splitseps.splitAtSeparators(
+                    i_raw_author, author_dividers
+                )
+                for a in raw_authors_split:
+                    cleaned_a = html.escape(a).strip()
+                    self.authors.append(cleaned_a)
+            else:
+                self.authors = [self.author]
+            site.processed_authors[i_raw_author] = self.authors
+        else:
+            self.author = self.metadata["author"]
+            self.authors = site.processed_authors[self.metadata["author"]]
+        self.author_email = self.metadata["email"]
+
 
 class Comic:
     """
@@ -561,6 +689,8 @@ class Comic:
         The original license name specified in the conf file.
     conf_c : dict
         Contents of the comic's config file.
+    copyright_statement : str
+        An HTML copyright statement.
     desc : str
         A description of the comic for the index.
     email : str
@@ -614,6 +744,11 @@ class Comic:
         Whether the comic is still updating or not.
     statuss : str
         status but wrapped in <strong> tags.
+
+    Methods
+    -------
+    populate:
+        Add initial information to a comic category.
     """
 
     __slots__ = [
@@ -634,6 +769,7 @@ class Comic:
         "chars_fn",
         "clicense",
         "conf_c",
+        "copyright_statement",
         "desc",
         "email",
         "fbd_link",
@@ -678,6 +814,103 @@ class Comic:
         self.category_escaped = html.escape(category)
         self.publicdomain = publicdomain
 
+    def populate(self, site, comics_base):
+        """
+        Add initial information to a comic category.
+
+        Parameters
+        ----------
+        site : Site
+            The root comic :class:`Site`.
+        comics_base : list of Strip
+            Strips known to Springheel.
+        """
+        self.known_pages = [
+            page.page_padded for page in comics_base if page.category == self.category
+        ]
+        self.known_pages_raw = [
+            page.page for page in comics_base if page.category == self.category
+        ]
+        if set([self.known_pages_raw.count(item) for item in self.known_pages_raw]) != {
+            1
+        }:
+            dupes = set(
+                [
+                    item
+                    for item in self.known_pages_raw
+                    if self.known_pages_raw.count(item) > 1
+                ]
+            )
+            logging.warning(
+                _("Duplicate page numbers found in category {cat}: {dupes}").format(
+                    cat=self.category, dupes=", ".join(dupes)
+                )
+            )
+        self.known_pages_real = [
+            page.page_real_num for page in comics_base if page.category == self.category
+        ]
+        self.known_pages.sort(key=springheel.mixNum)
+        self.known_pages_raw.sort()
+        self.known_pages_real.sort(key=springheel.realMixNum)
+        self.last_page, self.first_page = springheel.checkExtremes(self.known_pages)
+        raw_author = self.conf_c["author"]
+        self.author = html.escape(raw_author)
+        author_dividers = {"&", "/", "+"}
+        if any([div in raw_author for div in author_dividers]):
+            raw_authors_split = springheel.splitseps.splitAtSeparators(
+                raw_author, author_dividers
+            )
+            for a in raw_authors_split:
+                cleaned_a = html.escape(a).strip()
+                if cleaned_a not in site.authors:
+                    site.authors.append(cleaned_a)
+                if cleaned_a not in self.authors:
+                    self.authors.append(cleaned_a)
+        else:
+            if self.author not in site.authors:
+                site.authors.append(self.author)
+            if self.author not in self.authors:
+                self.authors.append(self.author)
+        for key in {
+            "email",
+            "header",
+            "banner",
+            "language",
+            "mode",
+            "status",
+            "license_uri",
+            "about",
+        }:
+            try:
+                setattr(self, key, self.conf_c[key])
+            except KeyError:
+                logging.debug(
+                    _("No value for {key} in the config file for {cat}.").format(
+                        key=key, cat=self.category
+                    )
+                )
+                continue
+        self.clicense = self.conf_c["license"]
+        try:
+            if (
+                self.conf_c["license"].lower() == "public domain"
+                or "publicdomain" in self.conf_c["license_uri"]
+            ):
+                self.publicdomain = True
+        except KeyError:
+            self.publicdomain = False
+        try:
+            # Just to be absolutely sure the slug really is safe.
+            self.slug = springheel.slugurl.slugify_url(self.conf_c["slug"])
+        except KeyError:
+            self.slug = springheel.slugurl.slugify_url(self.category)
+        if self.conf_c["chapters"]:
+            self.chapters = True
+            self.chapters_file = os.path.join(site.i_path, self.conf_c["chapters"])
+        else:
+            self.chapters = False
+        self.desc = html.escape(self.conf_c["desc"])
+
 
 class Chapter:
     """
@@ -813,6 +1046,8 @@ class EXpage:
         The generated HTML for the extras page.
     """
 
+    __slots__ = ["headings", "content"]
+
     def __init__(self):
         """Constructor for the EXpage class."""
         self.headings = []
