@@ -21,9 +21,12 @@
 
 import json, os, shutil
 from springheel.classes import EXpage
+import springheel.acopy
 from typing import Tuple
 import logging
 
+copyfile = springheel.acopy.wrap(shutil.copyfile)
+
 
 def duH(size_in_b: int, decimal_separator: str) -> str:
     """
@@ -108,6 +111,7 @@ def gen_extra(
     extras = EXpage()
     extra_elements = []
     sorted_j = sorted(j.items())
+    copy_queue = []
     for cat, elements in sorted_j:
         extras.headings.append(cat)
         subhead = "<h2>{cat}</h2>".format(cat=cat)
@@ -123,8 +127,11 @@ def gen_extra(
                             title=el["title"], image=image, width=width, height=height
                         )
                     )
-                    shutil.copy(
-                        os.path.join(i_path, image), os.path.join(o_path, image)
+                    copy_queue.append(
+                        {
+                            "source": os.path.join(i_path, image),
+                            "output": os.path.join(o_path, image),
+                        }
                     )
                 images = "".join(images)
                 el_template = """<figure>{images}<figcaption>{image_s}{desc}</figcaption></figure>""".format(
@@ -150,9 +157,11 @@ def gen_extra(
                             path=fil["path"], link=fil["link"], size=fsize
                         )
                     )
-                    shutil.copy(
-                        inp_file,
-                        os.path.join(o_path, fil["path"]),
+                    copy_queue.append(
+                        {
+                            "source": inp_file,
+                            "output": os.path.join(o_path, fil["path"]),
+                        }
                     )
                 fils = "".join(fils)
                 el_template = "<p>{desc}</p><ul>{fils}</ul>".format(
@@ -162,4 +171,18 @@ def gen_extra(
             extra_elements.append(elem)
     extra_combined = "\n".join(extra_elements)
     extras.content = extra_combined
-    return extras, j
+    return extras, j, copy_queue
+
+
+async def copyExtras(copy_queue: list) -> None:
+    """
+    Asynchronously copy items for the extras page.
+
+    Parameters
+    ----------
+    copy_queue : list of dict
+        A dictionary indicating the original paths and output paths of
+        files to be copied.
+    """
+    for queued_file in copy_queue:
+        await copyfile(queued_file["source"], queued_file["output"])
